"""Color definitions for graphics objects.

This module provides a dictionary of predefined RGB color values
for use with Pivy graphics objects. Colors are specified as tuples
of three float values in the range [0.0, 1.0] representing red, green, and blue.
"""

COLORS = {
    "black":  (0., 0., 0.),
    "white":  (1., 1., 1.),
    "grey":   (.5, .5, .5),
    "red":    (1., 0., 0.),
    "blue":   (0., 0., 1.),
    "green":  (0., 1., 1.),
    "yellow": (0., 1., 0.)
}
